ALIBABA_API_URL = 'http://gw.api.alibaba.com/openapi/param2/2/portals.open/%(api_call)s/%(api_key)s?%(call_parameters)s'

ALIBABA_API_CALLS = {
    'list': 'api.listPromotionProduct',
    'details': 'api.getPromotionProductDetail',
    'links': 'api.getPromotionLinks'
}

ALIBABA_API_PARAMS = {
    'list': [
        'fields',
        'keywords',
        'categoryId',
        'originalPriceFrom',
        'originalPriceTo',
        'volumeFrom',
        'volumeTo',
        'pageNo',
        'pageSize',
        'sort',
        'startCreditScore',
        'endCreditScore'
    ],
    'details': [
        'fields',
        'productId'
    ],
    'links': [
        'fields',
        'trackingId',
        'urls'
    ]
}

ALIBABA_API_FIELDS = {
    'list': [
        'totalResults',
        'productId',
        'productTitle',
        'productUrl',
        'imageUrl',
        'originalPrice',
        'salePrice',
        'discount',
        'evaluateScore',
        'commission',
        'commissionRate',
        '30daysCommission',
        'volume',
        'packageType',
        'lotNum',
        'validTime'
    ],
    'details': [

        'productId',
        'productTitle',
        'productUrl',
        'imageUrl',
        'originalPrice',
        'salePrice',
        'discount',
        'evaluateScore',
        'commission',
        'commissionRate',
        '30daysCommission',
        'volume',
        'packageType',
        'lotNum',
        'validTime',
        'storeName',
        'storeUrl'
    ],
    'links': [
        'totalResults',
        'trackingId',
        'publisherId',
        'url',
        'promotionUrl'
    ]
}

ALIBABA_API_CATEGORIES = {
    3: 'Apparel & Accessories',
    34: 'Automobiles & Motorcycles',
    1501: 'Baby Products',
    66: 'Beauty & Health',
    7: 'Computer & Networking',
    13: 'Construction & Real Estate',
    44: 'Consumer Electronics',
    100008578: 'Customized Products',
    5: 'Electrical Equipment & Supplies',
    502: 'Electronic Components & Supplies',
    2: 'Food',
    1503: 'Furniture',
    200003655: 'Hair & Accessories',
    42: 'Hardware',
    15: 'Home & Garden',
    6: 'Home Appliances',
    200003590: 'Industry & Business',
    36: 'Jewelry & Watch',
    39: 'Lights & Lighting',
    1524: 'Luggage & Bags',
    21: 'Office & School Supplies',
    509: 'Phones & Telecommunications',
    30: 'Security & Protection',
    322: 'Shoes',
    200001075: 'Special Category',
    18: 'Sports & Entertainment',
    1420: 'Tools',
    26: 'Toys & Hobbies',
    1511: 'Watches'
}

ALIBABA_API_ERROR_CODES = {
    'list': {
        20010000: 'Call succeeds',
        20020000: 'System Error',
        20030000: 'Unauthorized transfer request',
        20030010: 'Required parameters',
        20030020: 'Invalid protocol format',
        20030030: 'API version input parameter error',
        20030040: 'API name space input parameter error',
        20030050: 'API name input parameter error',
        20030060: 'Fields input parameter error',
        20030070: 'Keyword input parameter error',
        20030080: 'Category ID input parameter error',
        20030090: 'Tracking ID input parameter error',
        20030100: 'Commission rate input parameter error',
        20030110: 'Original Price input parameter error',
        20030120: 'Discount input parameter error',
        20030130: 'Volume input parameter error',
        20030140: 'Page number input parameter error',
        20030150: 'Page size input parameter error',
        20030160: 'Sort input parameter error',
        20030170: 'Credit Score input parameter error'
    },
    'details': {
        20010000: 'Call succeeds',
        20020000: 'System Error',
        20030000: 'Unauthorized transfer request',
        20030010: 'Required parameters',
        20030020: 'Invalid protocol format',
        20030030: 'API version input parameter error',
        20030040: 'API name space input parameter error',
        20030050: 'API name input parameter error',
        20030060: 'Fields input parameter error',
        20030070: 'Product ID input parameter error'
    },
    'links': {
        20010000: 'Call succeeds',
        20020000: 'System Error',
        20030000: 'Unauthorized transfer request',
        20030010: 'Required parameters',
        20030020: 'Invalid protocol format',
        20030030: 'API version input parameter error',
        20030040: 'API name space input parameter error',
        20030050: 'API name input parameter error',
        20030060: 'Fields input parameter error',
        20030070: 'Tracking ID input parameter error',
        20030080: 'URL input parameter error or beyond the maximum number of the URLs'
    }
}



import sys

from .config import ALIBABA_API_ERROR_CODES, ALIBABA_API_CALLS

__all__ = ['AliExpressAPICallError', 'NetworkError', '_e']


class AliExpressAPICallError(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self)
        self.args = args
        self.call = ALIBABA_API_CALLS[kwargs.pop('call', None)]
        self.code = kwargs.pop('code', None)
        self.msg = ALIBABA_API_ERROR_CODES[self.code]

    def __str__(self):  # pragma: no cover
        if self.code is not None:
            return '%(code)s: %(msg)s. Call: %(call)s' % self.__dict__
        return Exception.__str__(self)


class NetworkError(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self)
        self.args = args
        self.call = ALIBABA_API_CALLS[kwargs.pop('call', None)]
        self.code = kwargs.pop('code', None)
        self.msg = kwargs.pop('msg', None)

    def __str__(self):  # pragma: no cover
        if self.code is not None:
            return '%(code)s: %(msg)s in %(call)s' % self.__dict__
        return Exception.__str__(self)


def _e(*args, **kwargs):
    """
    Returns an exception of type ``error_class`` based on an instance of
    :class:`AWSError`  all relevant information appended.
    """
    exc = sys.exc_info()[1]
    error = AliExpressAPICallError(*args)
    error.call = exc.call
    error.code = exc.code
    error.msg = exc.msg if exc.msg else None
    return error



import logging
import urllib
import json

from .config import ALIBABA_API_URL, ALIBABA_API_CALLS, ALIBABA_API_FIELDS, ALIBABA_API_PARAMS, ALIBABA_API_CATEGORIES
from .errors import *

LOGGER = logging.getLogger(__name__)


class AliExpress(object):
    def __init__(self, api_key, affiliate_id=None, digital_signature=None):
        self.api_key = api_key
        self.affiliate_id = affiliate_id
        self.digital_signature = digital_signature

    def get_product_list(self, fields, keywords, **kwargs):
        if not isinstance(fields, list):
            raise ValueError('Parameter %s must be a list', 'fields')

        for field in fields:
            if field not in ALIBABA_API_FIELDS['list']:
                raise ValueError('Field %s must be in %s' % (field, str(ALIBABA_API_FIELDS['list'])))

        params = {
            'fields': ','.join(fields),
            'keywords': keywords
        }

        for param, value in kwargs.items():
            if param not in ALIBABA_API_PARAMS['list']:
                raise ValueError('Parameter %s must be in %s' % (param, str(ALIBABA_API_PARAMS['list'])))

            if param == 'categoryId' and value not in ALIBABA_API_CATEGORIES.keys():
                raise ValueError('Category %s must be in %s' % (value, str(ALIBABA_API_CATEGORIES)))

            params[param] = value

        response = self._make_call('list', params)

        if 'result' in response:
            return response['result']
        else:
            return None

    def get_product_details(self, fields, product_id):
        if not isinstance(fields, list):
            raise ValueError('Parameter %s must be a list', 'fields')

        for field in fields:
            if field not in ALIBABA_API_FIELDS['details']:
                raise ValueError('Field %s must be in %s' % (field, str(ALIBABA_API_FIELDS['details'])))

        params = {
            'fields': ','.join(fields),
            'productId': product_id
        }

        response = self._make_call('details', params)

        if 'result' in response:
            return response['result']
        else:
            return None

    def get_promotion_links(self, fields, urls, tracking_id=None):
        if tracking_id:
            self.affiliate_id = tracking_id

        if self.affiliate_id is None:
            raise ValueError('Affiliate tracking id is not present')

        if not isinstance(fields, list):
            raise ValueError('Parameter %s must be a list', 'fields')

        if not isinstance(urls, list):
            raise ValueError('Parameter %s must be a list', 'urls')

        for field in fields:
            if field not in ALIBABA_API_FIELDS['links']:
                raise ValueError('Field %s must be in %s' % (field, str(ALIBABA_API_FIELDS['links'])))

        params = {
            'fields': ','.join(fields),
            'trackingId': self.affiliate_id,
            'urls': ','.join(urls)
        }

        response = self._make_call('links', params)

        if 'result' in response:
            return response['result']
        else:
            return None

    def _make_call(self, call, params):
        url = ALIBABA_API_URL % {
            'api_call': ALIBABA_API_CALLS[call],
            'api_key': self.api_key,
            'call_parameters': urllib.urlencode(params)
        }
        LOGGER.info('Perform API request url: %s' % url)
        response = urllib.urlopen(url)

        if response.code != 200:
            raise NetworkError(call=call, code=response.code, msg=json.loads(response.read())['error_message'])

        response_json = json.loads(response.read())

        if response_json['errorCode'] != 20010000:
            raise _e(call=call, code=response_json['errorCode'])

        return response_json



try:
    import multiprocessing
except ImportError:
    pass


from setuptools import setup

setup(
    name='aliexpress-api-client',
    version='0.1.0',
    description='AliExpress API Client',
    url='https://github.com/kronas/python-aliexpress-api-client',
        
    author='Kronas',
    author_email='kronas.sw@gmail.com',
    license='MIT',

    packages=['aliexpress_api_client'],

    install_requires=[],

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    zip_safe=False,
)
